﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace MultiWiiSimulator
{
    public class PID
    {
        // the PID coefficients (stay constant)
        public double Kp, Ki, Kd;

        // the individual computed components
        public double P, I, D;

        // inputs
        public double Error;

        // output
        public double Output;

        private double[] history;
        private double lastError;

        public PID(double Kp = 1.0, double Ki = 0.0, double Kd = 0.0, int history_size = 5)
        {
            this.Kp = Kp;
            this.Ki = Ki;
            this.Kd = Kd;
            history = new double[history_size];
        }

        public double Compute(double new_error)
        {
            lastError = Error;
            Error = new_error;

            // compute components
            P = Kp * Error;

            I = 0;
            for (int i = 0, c = history.Length; i < c; i++)
                I += history[i];
            I *= Ki;

            D = Kd * (Error-lastError);

            // update history
            for (int i = 0; i < history.Length - 1; i++)
                history[i] = history[i + 1];
            history[history.Length-1] = Error;

            return Output = P + I - D;
        }

        public void DebugPrint(string name="")
        {
            Console.WriteLine(String.Format("{5}-PID E{0:0.00} -> [{1:0.00},{2:0.00},{3:0.00}] = {4:0.00}",
                Error, P, I, D, Output, name
            ));
        }
    }
}
