﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

namespace MultiWiiSimulator
{
    public partial class SimForm : Form
    {
        public Copter Copter;

        private Point altitudeLevelBottomLeft;
        private bool editingAltitude=false;

        private double kickForce = 0;
        private bool showActualVelocity = true;

        public SimForm()
        {
            InitializeComponent();
            //Copter.SetAltitude(125);
            Reset();
        }

        public void Reset()
        {
            Copter = new Copter();
            //Copter.VelocityKeeping = 3.5;
            VelocityTrack.Minimum = (int)-Copter.MaxVelocity*10;
            VelocityTrack.Maximum = (int)Copter.MaxVelocity * 10;
            VelocityTrack.TickFrequency = (int)Copter.MaxVelocity*10/15;
        }

        public void ToggleAltitudeHold()
        {
            Copter.AltitudeHold = !Copter.AltitudeHold;
            AltitudeHoldEnable.BackColor = Copter.AltitudeHold ? Color.Lime : Color.Gainsboro;
        }

        public void ToggleVelocityHold()
        {
            Copter.VelocityHold = !Copter.VelocityHold;
            VelocityHold.BackColor = Copter.VelocityHold ? Color.Lime : Color.Gainsboro;
        }

        public void UpdateControls()
        {
            Barometer.Value = (float)Copter.Baro;
            Altitude.Value = (float)Copter.Position.Z;
            Altitude.Value2 = (float)Copter.EstimatedPosition.Z;
            Thrust.Value = (float)Copter.Thrust;
            BarometerReading.Text = Copter.Baro.ToString("0.0")+"m";
            TrueAltitudeReading.Text = Copter.Position.Z.ToString("0.0") + "m";
            ThrustReading.Text = Copter.Thrust.ToString("0.0");

            Velocity.Value = (float)Copter.Velocity.Z;
            Velocity.Value2 = (float)Copter.EstimatedVelocity.Z;
            VelocityReading.Text = (showActualVelocity ? Copter.Velocity : Copter.EstimatedVelocity).Z.ToString("0.0") + " m/s";

            Accelleration.Value = (float)Copter.Accelleration.Z;
            //Accelleration.Value2 = (float)Copter.Acc.Z;
            AccellerationReading.Text =Copter.Accelleration.Z.ToString("0.0") + " m/s";
            // AccellerationReading.Text = (showActualAccelleration ? Copter.Accelleration : Copter.Acc).Z.ToString("0.0") + " m/s";

            ACC.Value = (float)Copter.Acc.Z;
            ACCReading.Text = Copter.Acc.Z.ToString("0.0");

            if(altitudeLevelBottomLeft.IsEmpty)
                altitudeLevelBottomLeft = new Point(AltitudeLevel.Left, AltitudeLevel.Bottom);
            AltitudeLevel.Height = (int)Copter.Position.Z;
            AltitudeLevel.Top = altitudeLevelBottomLeft.Y - AltitudeLevel.Height;

            if (Copter.AltitudeHold || Copter.VelocityHold)
            {
                ThrottleTrack.Value = (int)Copter.Throttle;
                ThrottleTrack.Enabled = ThrottleValue.Enabled = false;
                if(!editingAltitude)
                    DesiredAltitude.Text = Copter.AltitudeKeeping.ToString("0.0");
                AltitudeHoldMarker.Top = altitudeLevelBottomLeft.Y - (int)Copter.AltitudeKeeping;
            }
            else
            {
                Copter.Throttle =ThrottleTrack.Value;
                ThrottleTrack.Enabled = ThrottleValue.Enabled = true;
                DesiredAltitude.Text = "-";
            }
            AltitudeHoldMarker.Visible = Copter.AltitudeHold;
            ThrottleValue.Text = Copter.Throttle.ToString();

            if (Copter.VelocityHold)
            {
                if (Copter.AltitudeHold)
                {
                    VelocityTrack.Value = (int)(Copter.VelocityKeeping * 10);
                    VelocityTrackValue.Text = Copter.VelocityKeeping.ToString("0.0");
                }
                else
                {
                    VelocityTrackValue.Text = Copter.VelocityKeeping.ToString("0.0");
                    Copter.VelocityKeeping = (double)VelocityTrack.Value/10;
                }
            }
            else
            {
                VelocityTrackValue.Text = String.Empty;
                VelocityTrackValue.Enabled = VelocityTrack.Enabled = false;

            }
            VelocityTrack.Enabled = Copter.VelocityHold && !Copter.AltitudeHold;
            VelocityHold.Enabled = !Copter.AltitudeHold;
            VelocityHold.BackColor = (Copter.VelocityHold || Copter.AltitudeHold) ? Color.Lime : SystemColors.Control;

            if ((Copter.Ticks % 30) == 0)
            {
                //Copter.AltitudePID.DebugPrint("Altitude");
                if(Copter.VelocityHold)
                    Copter.VelocityPID.DebugPrint("Velocity");
            }
        }

        private void updateTimer_Tick(object sender, EventArgs e)
        {
            if (kickForce != 0)
            {
                Copter.Position.Z += kickForce/Copter.CyclesPerSecond;
                kickForce *= 0.8;
                if (Math.Abs(kickForce) < 0.1)
                    kickForce = 0;
            }
            Copter.millis += updateTimer.Interval;
            Copter.Tick();
            UpdateControls();
        }

        private void PauseSimulation(object sender, EventArgs e)
        {
            updateTimer.Enabled = !updateTimer.Enabled;
            Pause.BackColor = updateTimer.Enabled ? Color.Lime : Color.Gainsboro;
        }

        private void AltitudeHoldEnable_Click(object sender, EventArgs e)
        {
            ToggleAltitudeHold();
        }

        private void button1_Click(object sender, EventArgs e)
        {
            Reset();
        }

        private void DesiredAltitude_Leave(object sender, EventArgs e)
        {
            try
            {
                Copter.AltitudeKeeping = Double.Parse(DesiredAltitude.Text);
            }
            catch (FormatException)
            {
                DesiredAltitude.Text = String.Empty;
            }
            editingAltitude = false;
        }

        private void DesiredAltitude_Enter(object sender, EventArgs e)
        {
            editingAltitude = true;
        }

        private void VelocityHold_Click(object sender, EventArgs e)
        {
            ToggleVelocityHold();
        }

        private void KickCopter_Click(object sender, EventArgs e)
        {
            kickForce = KickForceTrack.Value;
        }

        private void KickForceTrack_ValueChanged(object sender, EventArgs e)
        {
            KickForceValue.Text = ((double)KickForceTrack.Value / 10).ToString("0.0");
        }

        private void ToggleVelocityDisplay(object sender, EventArgs e)
        {
            showActualVelocity = !showActualVelocity;
            VelocityDisplayMode.Text = showActualVelocity ? "ACTUAL" : "EST.";
        }

        private void Velocity_Click(object sender, EventArgs e)
        {

        }
    }
}
