﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.Runtime.InteropServices;

namespace MultiWii
{
    public partial class AltHoldTuningDialog : Form
    {
        [DllImport("user32.dll")]
        private static extern long LockWindowUpdate(long Handle);

        public string NumberFormat = "0.0";

        private bool hasChanged = true;
        public bool HasChanged
        {
            get { bool hc = hasChanged; hasChanged = false; return hc; }
            set { hasChanged = value; }
        }

        #region Position Loop
        public double PositionKp
        {
            get
            {
                return (double)PositionKpTrack.Value / 100;
            }
            set
            {
                PositionKpTrack.Value = (int)(value * 100);
                hasChanged = true;
            }
        }

        public double PositionKi
        {
            get
            {
                return (double)PositionKiTrack.Value / 100;
            }
            set
            {
                PositionKiTrack.Value = (int)(value * 100);
                hasChanged = true;
            }
        }

        public double PositionKd
        {
            get
            {
                return (double)PositionKdTrack.Value / 100;
            }
            set
            {
                PositionKdTrack.Value = (int)(value * 100);
                hasChanged = true;
            }
        }

        private double position_output_max = 100.0;
        public double PositionControlMax
        {
            get
            {
                return position_output_max;
            }
            set
            {
                position_output_max = value;
                updateOutputCenterBar(PositionOutputBar, position_output/position_output_max);
                PositionOutputValue.Text = position_output.ToString(NumberFormat);
                hasChanged = true;
            }
        }

        private double position_output;
        public double PositionControlOutput
        {
            get
            {
                return position_output;
            }
            set
            {
                position_output = value;
                updateOutputCenterBar(PositionOutputBar, position_output/position_output_max);
                PositionOutputValue.Text = position_output.ToString(NumberFormat);
            }
        }
        #endregion

        #region Velocity Loop
        public double VelocityKp
        {
            get
            {
                return (double)VelocityKpTrack.Value / 100;
            }
            set
            {
                VelocityKpTrack.Value = (int)(value * 100);
                hasChanged = true;
            }
        }

        public double VelocityKi
        {
            get
            {
                return (double)VelocityKiTrack.Value / 100;
            }
            set
            {
                VelocityKiTrack.Value = (int)(value * 100);
                hasChanged = true;
            }
        }

        public double VelocityKd
        {
            get
            {
                return (double)VelocityKdTrack.Value / 100;
            }
            set
            {
                VelocityKdTrack.Value = (int)(value * 100);
                hasChanged = true;
            }
        }

        public double GravityFeedForward
        {
            get
            {
                return (double)GravityFFTrack.Value / 10;
            }
            set
            {
                GravityFFTrack.Value = (int)(value * 10);
                hasChanged = true;
            }
        }

        private double velocity_output_max = 100.0;
        public double VelocityControlMax
        {
            get
            {
                return velocity_output_max;
            }
            set
            {
                velocity_output_max = value;
                updateOutputCenterBar(VelocityOutputBar, velocity_output/velocity_output_max);
                VelocityOutputValue.Text = velocity_output.ToString(NumberFormat);
                hasChanged = true;
            }
        }
        
        private double velocity_output;
        public double VelocityControlOutput
        {
            get
            {
                return velocity_output;
            }
            set
            {
                velocity_output = value;
                updateOutputCenterBar(VelocityOutputBar, velocity_output/velocity_output_max);
                VelocityOutputValue.Text = velocity_output.ToString(NumberFormat);
            }
        }
        #endregion

        #region Throttle
        public double ThrottleLowPass
        {
            get
            {
                return (double)ThrottleAlphaTrack.Value / 1000;
            }
            set
            {
                ThrottleAlphaTrack.Value = (int)(value * 1000);
                hasChanged = true;
            }
        }

        private int throttle_output;
        public int ThrottleOutput
        {
            get
            {
                return throttle_output;
            }
            set
            {
                throttle_output = value;
                updateOutputPositiveBar(ThrottleOutputBar, ((double)throttle_output-1000)/1000);
                ThrottleOutputValue.Text = throttle_output.ToString();
            }
        }
        #endregion

        private bool running;
        public bool Running
        {
            get
            {
                return running;
            }
            set
            {
                running = value;
                stopButton.Enabled = running;
            }
        }



        public AltHoldTuningDialog()
        {
            InitializeComponent();
            PositionKpTrack.Tag = PositionKpValue;
            PositionKiTrack.Tag = PositionKiValue;
            PositionKdTrack.Tag = PositionKdValue;
            VelocityKpTrack.Tag = VelocityKpValue;
            VelocityKiTrack.Tag = VelocityKiValue;
            VelocityKdTrack.Tag = VelocityKdValue;
            
            // set initial values
            Running = false;
            PositionPID(4.5, 0.5, 0);
            VelocityPID(4.5, 0.5, 0, 50.0);
            PositionControlOutput = 0.0;
            VelocityControlOutput = 0.0;
            ThrottleOutput = 1200;
            ThrottleLowPass = 0.95;
       }

        public void PositionPID(double Kp, double Ki, double Kd)
        {
            PositionKp = Kp;
            PositionKi = Ki;
            PositionKd = Kd;
        }

        public void VelocityPID(double Kp, double Ki, double Kd)
        {
            VelocityKp = Kp;
            VelocityKi = Ki;
            VelocityKd = Kd;
        }

        public void VelocityPID(double Kp, double Ki, double Kd, double Gravityff)
        {
            VelocityKp = Kp;
            VelocityKi = Ki;
            VelocityKd = Kd;
            GravityFeedForward = Gravityff;
        }

        private void updateOutputCenterBar(PictureBox bar, double value)
        {
            LockWindowUpdate((long)Handle);
            value = Math.Min(Math.Max(value, -1.0), 1.0);
            bar.Top = OutputBarBackground.Top + OutputBarBackground.Height / 2;
            int bh = (int)(-OutputBarBackground.Height * value/2);
            if(bh<0) {
                bar.Height = -bh;
                bar.Top -= -bh;
            } else
                bar.Height = bh;
            LockWindowUpdate(0);
        }

        private void updateOutputPositiveBar(PictureBox bar, double value)
        {
            LockWindowUpdate((long)Handle);
            value = Math.Min(Math.Max(value, -1.0), 1.0);
            bar.Top = OutputBarBackground.Bottom - (int)(OutputBarBackground.Height * value);
            bar.Height = OutputBarBackground.Bottom - bar.Top;
            LockWindowUpdate(0);
        }

        private void OnPIDTrackBarChanged(object sender, EventArgs e)
        {
            TrackBar tracker = sender as TrackBar;
            if (tracker != null && tracker.Tag!=null)
            {
                TextBox tracker_value = tracker.Tag as TextBox;
                if (tracker_value != null)
                    tracker_value.Text = ((double)tracker.Value / 100).ToString(NumberFormat);
                hasChanged = true;
            }
        }

        private void GravityFFTrack_ValueChanged(object sender, EventArgs e)
        {
            GravityFFValue.Text = ((double)GravityFFTrack.Value / 10).ToString(NumberFormat);
            hasChanged = true;
        }

        private void ThrottleAlphaTrack_ValueChanged(object sender, EventArgs e)
        {
            ThrottleAlphaValue.Text = ((double)ThrottleAlphaTrack.Value / 1000).ToString(NumberFormat);
            hasChanged = true;
        }

        private void stopButton_Click(object sender, EventArgs e)
        {
            Running = false;
        }


    }
}
