﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace MultiWii
{
    public class PitchRollZ
    {
        public Int16 Pitch;
        public Int16 Roll;
        public Int16 Z;

        public static PitchRollZ ReadFromArray(byte[] array, int offset, int count_of_words)
        {
            PitchRollZ prz = new PitchRollZ();
            prz.Pitch = (Int16)( (UInt16)array[offset] | ((UInt16)array[offset + 1] << 8) );
            offset += 2; if (--count_of_words <= 0) return prz; 
            prz.Roll = (Int16)((UInt16)array[offset] | ((UInt16)array[offset + 1] << 8));
            offset += 2; if (--count_of_words <= 0) return prz;
            prz.Z = (Int16)((UInt16)array[offset] | ((UInt16)array[offset + 1] << 8));
            return prz;
        }

        public double VarianceFrom(PitchRollZ rhs)
        {
            double p = rhs.Pitch - Pitch;
            double r = rhs.Roll - Roll;
            double z = rhs.Z - Z;
            return Math.Sqrt(p * p + r * r + z * z);
        }
    }

    public class PitchRollYaw
    {
        public Int16 Pitch;
        public Int16 Roll;
        public Int16 Yaw;

        public static PitchRollYaw ReadFromArray(byte[] array, int offset, int count_of_words)
        {
            PitchRollYaw prz = new PitchRollYaw();
            prz.Pitch = (Int16)((UInt16)array[offset] | ((UInt16)array[offset + 1] << 8));
            offset += 2; if (--count_of_words <= 0) return prz;
            prz.Roll = (Int16)((UInt16)array[offset] | ((UInt16)array[offset + 1] << 8));
            offset += 2; if (--count_of_words <= 0) return prz;
            prz.Yaw = (Int16)((UInt16)array[offset] | ((UInt16)array[offset + 1] << 8));
            return prz;
        }

        public double VarianceFrom(PitchRollYaw rhs)
        {
            double p = rhs.Pitch - Pitch;
            double r = rhs.Roll - Roll;
            double y = rhs.Yaw - Yaw;
            return Math.Sqrt(p * p + r * r + y * y);
        }
    }

    public class SensorData
    {
        public byte version;   // multiwii firmware version
        
        public PitchRollZ Acc;
        public PitchRollYaw Gyro;
        public PitchRollYaw Mag;

        public Int16 VerticalAcc;
        public Int16 Altitude;
        public Int16 Heading;

        public Int16[] Servo;
        public Int16[] Motor;
        public Int16[] Channels;

        public Int16 CycleTime;
        public Int32 TimeStamp;

        public static Int16[] ReadInt16s(byte[] array, int offset, int count_of_words)
        {
            Int16[] oa = new Int16[count_of_words];
            for(int i=0,j=offset; i<count_of_words; i++,j+=2)
                oa[i] = (Int16)((UInt16)array[j] | ((UInt16)array[j + 1] << 8));
            return oa;            
        }

        public static Int16 ReadInt16(byte[] array, int offset)
        {
            return (Int16)((UInt16)array[offset] | ((UInt16)array[offset + 1] << 8));
        }

        public static SensorData Read(byte[] array, int offset)
        {
            SensorData data = new SensorData();
            data.version = array[offset++];
            data.Acc = PitchRollZ.ReadFromArray(array, offset, 3); offset += 6;
            data.Gyro = PitchRollYaw.ReadFromArray(array, offset, 3); offset += 6;
            data.Mag = PitchRollYaw.ReadFromArray(array, offset, 3); offset += 6;
            data.Altitude = ReadInt16(array, offset); offset += 2;
            data.Heading = ReadInt16(array, offset); offset += 2;
            data.Servo = ReadInt16s(array, offset, 4); offset += 8;
            data.Motor = ReadInt16s(array, offset, 8); offset += 16;
            data.Channels = ReadInt16s(array, offset, 8); offset += 16;
            offset += 2;
            data.CycleTime = ReadInt16(array, offset); offset += 2;
            return data;
        }

        public static SensorData ReadBrief(byte[] array, int offset)
        {
            // SENTENCE (no spaces, 2 byte signed ints):
            // "L" ACC[XYZ] GYRO[XYZ] MAG[XYZ] ALT HEADING MILLIS_H:MILLIS_L "\n"
            SensorData data = new SensorData();
            data.Acc = PitchRollZ.ReadFromArray(array, offset, 3); offset += 6;
            data.Gyro = PitchRollYaw.ReadFromArray(array, offset, 3); offset += 6;
            data.Mag = PitchRollYaw.ReadFromArray(array, offset, 3); offset += 6;
            data.VerticalAcc = ReadInt16(array, offset); offset += 2;
            data.Altitude = ReadInt16(array, offset); offset += 2;
            data.Heading = ReadInt16(array, offset); offset += 2;
            //data.Servo = ReadInt16s(array, offset, 4); offset += 8;
            //data.Motor = ReadInt16s(array, offset, 8); offset += 16;
            //data.Channels = ReadInt16s(array, offset, 8); offset += 16;
            //data.CycleTime = ReadInt16(array, offset); offset += 2;
            Int16 H = ReadInt16(array, offset); offset += 2;
            Int16 L = ReadInt16(array, offset); offset += 2;
            data.TimeStamp = (H << 16) + L;
            return data;
        }
    }
}
